#include <chrono>
#include <functional> // 'std::bind'...
#include <iostream>
#include <thread>

#include "Short_dead_lock.h"

void Short_dead_lock::f() {
	std::cout << "Start of 'f()' in thread: " << std::this_thread::get_id() << std::endl;
	// _g(); // '_g' *MUST* be started in another thread to avoid self-deadlock: 
	std::thread g(&Short_dead_lock::_g, this); // This starts 'g'...

	// std::this_thread::sleep_for(std::chrono::milliseconds(5000)); // Wait for 5s...
	std::chrono::seconds my_duration(5);
	using namespace std::chrono_literals; // '<chrono>' reserved suffixes...
	std::chrono::milliseconds my_duration_ = 5000ms;
	std::this_thread::sleep_for(5000ms); // Wait for 5s...

	__stop = true; // Required for ver. 2
	// Release '_g':
	_my_condition_variable.notify_all(); // Required for ver. 1 and ver. 2...
	g.join(); // Wait for 'g' thread to complete...
	std::cout << "End of 'f()'" << std::endl;
}

void Short_dead_lock::_g() {
	std::cout << "Start of '_g()' in thread: " << std::this_thread::get_id() << std::endl;
	std::unique_lock<std::mutex> lock(_my_mutex);
	// Ver. 1:
	//_my_condition_variable.wait(lock);
	// Ver. 2:
	_my_condition_variable.wait(lock, std::bind(&Short_dead_lock::_stop, this));
	std::cout << "End of '_g()'" << std::endl;
}

int main(int argc, char** argv) {
	Short_dead_lock sdl;
	sdl.f();

	return 0;
}

